{% extends "charme_base.html" %}

{% load i18n %}

{% block main %}

<h2>Logged out</h2>

<p>Thanks for spending some quality time with the Web site today.</p>

<p><a href="{% url 'login' %}">Log in again</a></p>

{% endblock %}
