.buttons
{
display: inline-block;
background: #eeeeee; /* Old browsers */
background: -moz-linear-gradient(top, #eeeeee 0%, #eeeeee 100%); /* FF3.6+ */
background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#eeeeee), color-stop(100%,#eeeeee)); /* Chrome,Safari4+ */
background: -webkit-linear-gradient(top, #eeeeee 0%,#eeeeee 100%); /* Chrome10+,Safari5.1+ */
background: -o-linear-gradient(top, #eeeeee 0%,#eeeeee 100%); /* Opera11.10+ */
background: -ms-linear-gradient(top, #eeeeee 0%,#eeeeee 100%); /* IE10+ */
filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#eeeeee', endColorstr='#eeeeee',GradientType=0 ); /* IE6-9 */
background: linear-gradient(top, #eeeeee 0%,#eeeeee 100%); /* W3C */
border: 1px solid #a1a1a1;
padding: 0 2em;
margin: 0.5em;
font: bold 1em/2em Arial, Helvetica;
text-decoration: none;
color: #333;
-moz-border-radius: .2em;
-webkit-border-radius: .2em;
border-radius: .2em;
}

.buttons:before
{
float: left;
width: 1em;
#text-align: center;
#font-size: 1.7em;
#margin: 0 0.5em 0 -1em;
#padding: 0 .2em;
#pointer-events: none;
}

.dojoxGridCell {
	border-width: 0px !important;
	padding: 0px !important;
}

.dojoxGridCellFocus {
	border: 0px !important;
}

.dojoxGridHeader.dojoxGridCellFocus {
	border-bottom-color: transparent !important;
}

.content * div div.dojoxGridRowSelected {
	background-color: #FEBE47 !important;
}

.content table.dojoxGridRowTable {
	#opacity:0.52;
	background-color: transparent !important;
}

.content table td.dojoxGridCell {
	#opacity:0.52;
	background-color: transparent !important;
}

.content .accounts ul li {
	line-height: 0.0em;
}

.content .accounts table {
	width: auto;
	margin: 2.0em 0 0.2em 0px;
}

.content .accounts table td {
	height: 3.0em;
	padding: 0px 3px 0px 0px;
	border: solid 0px rgb(255, 255, 255);
	background-color: rgb(255, 255, 255);
}

.content .error ul {
	padding: 0px;
}

.content .error ul li {
	background: rgb(255, 255, 255);
	color: #FF0000;
}

.links {
		padding: 5px 5px 5px 5px;
		text-align: right;
}