{% extends "base_no_sidebar.html" %}

{% block extra_head %}
<link rel="stylesheet" href="{{ STATIC_URL }}/charme/css/charme.css" />
{% block extra_extra_head %}
{% endblock %}
{% endblock %}

{% block title %}

This is a BETA service

{% endblock %}

{% block navbar %}

<div class=links>
	{% if user.is_authenticated %}
		Hello {{ user.first_name }}
		[<a href="{% url 'profile' %}">Profile</a>] 
		[<a href="{% url 'user_signout' %}">Logout</a>]
	{% else %}
		[<a href="{% url 'login' %}">Login</a> or 
		<a href="{% url 'registration' %}">Register</a>]
	{% endif %}
</div>

{% endblock %}
