{% extends "charme_base.html" %}

{% load i18n %}

{% block main %}

<h2>Software Agent {{ uri }}</h2>

<div class=contactform id="agent">
	<div class=accounts>
		<table>
			<tr>
				<td><label>{% trans "Name:" noop %}</label></td>
				<td>{{ name }}</td>
			</tr>
		</table>
	</div>
</div>

{% endblock %}
