{% extends "charme_base.html" %} {% load i18n %} {% block main %}

<h2>Stop Following Resource</h2>

<form action="" method="post">
	{% csrf_token %}
	<p>Are you sure you want to stop following "{{ object }}"?</p>
	<p>
		<input type="submit" value="Confirm" />
		<input type="button" 
			onclick="location.href='{% url 'following-list' %}';"
			value="Cancel" />
	</p>
</form>

{% endblock %}

