<h1>Conditions of Use</h1>
<h2>Data Protection</h2>
<p>On registering with the CHARMe node, STFC will store your email
	address. The username you select will be made public. Your email will
	also be made public, if you have selected this option in your user
	profile.</p>
<p>The annotation data will be stored at STFC, however STFC are not
	responsible for the content of the annotation nor for any comments/
	views held therein.</p>
<p>Inappropriate content will be reported to the moderators. Users
	who abuse the Usage Policy Appendix A1.2 may have their accounts
	barred/ removed.</p>
<h2>Usage Policy</h2>
<p>By registering with the CHARMe node, you are agreeing to be held
	responsible for the information you provide and confirming that you
	accept and understand the following rules of use:</p>
<ul>
	<li>Annotations will be free from malicious, abusive or obscene
		comments/ links.</li>
	<li>Annotations will not contain any links to services relating to
		marketing, advertising or recruitment.</li>
	<li>All annotations reflect the views of the author of the
		annotation. STFC will not be held responsible for the content of any
		annotation.</li>
	<li>The author of an annotation retains the copyright of his/her
		annotation.</li>
	<li>Any user who attempts to cause technical problems for the
		system will be reported to the moderator and will be barred.</li>
	<li>STFC reserves the right to store the annotations and make
		available to other users according to this Usage Policy.</li>
	<li>The author grants STFC permission to, without changing
		content, translate the data to any medium or format for the purpose of
		future preservation and accessibility.</li>
</ul>
<p>Note that:</p>
<ul>
	<li>The CHARMe node is not responsible for maintaining the
		permanence of URL’s. The CHARMe node shall not monitor for permanence
		of URLs or broken links.</li>
	<li>STFC accepts no responsibility for the network connection
		between the CHARMe node and the CHARMe plugin.</li>
</ul>
<p>These rules are not exhaustive and the CHARMe moderators reserve
	the right to edit/delete annotations and/ or take other actions as
	necessary in order to maintain the operation and availability of the
	CHARMe system.</p>
