{% extends "registration/profile.html" %}

{% load i18n %}

{% block details %}

{% if user_form.non_field_errors %}
<div class=error>
	<ul>
		<li>{{ user_form.non_field_errors }}</li>
	</ul>
</div>
{% endif %}

<div class=contactform>
	<div class=accounts>
		<form method="post" action="{% url 'profile_change' %}">
			{% csrf_token %}
			<table>
				<tr>
					<td><label>{% trans "First name:" noop %}</label></td>
					<td>{{ user_form.first_name }}</td>
					<td class=error>{{ user_form.first_name.errors }}</td>
				</tr>
				<tr>
					<td><label>{% trans "Last name:" noop %}</label></td>
					<td>{{ user_form.last_name }}</td>
					<td class=error>{{ user_form.last_name.errors }}</td>
				</tr>
				<tr>
					<td><label>{% trans "Email:" noop %}</label></td>
					<td>{{ user_form.email }}</td>
					<td class=error>{{ user_form.email.errors }}</td>
				</tr>
				<tr>
					<td><label>{% trans "Show my email in annotations:" noop %}</label></td>
					<td>{{ user_profile_form.show_email }}</td>
					<td class=error>{{ user_profile_form.show_email.errors }}</td>
				</tr>
				<tr>
					<td></td>
					<td style="text-align: right;">
					<input type="submit" value="Update account">
					<input type="button"
					onclick="location.href='{% url 'profile' %}'"
					value="Cancel" /></td>
					<td></td>
				</tr>
			</table>
		</form>
	</div>
</div>

{% endblock %}
