{% extends "charme_base.html" %}

{% load i18n %}

{% block main %}

<div class="loginform">
	<h4>Dissociate Open ID from your account</h4>
	{% if msg %}
	<div class=error>
		<ul>
			<li>{{ msg }}</li>
		</ul>
	</div>
	{% endif %}
    <div class=accounts>
		<form action="" method="post">
		    {% csrf_token %}
		    <input type="hidden" name="next" value="{{ request.REQUEST.next }}" />
		    {{ form.openid_url }}
    		<table>
				<tr>
					<td><label> {% trans "Confirm to dissociate " noop %}{{ openid_url }}{% trans " from your account" noop %} </label></td>
				</tr>
				<tr>
					<td style="text-align: right;">
						<input type="submit" value="{% trans "Dissociate OpenID" %}">
						<input type="button"
						onclick="location.href='{% url 'profile' %}'"
						value="Cancel" /></td>
				</tr>
			</table>	
		</form>
	</div>	
</div>
	
{% endblock %}
