{% extends "charme_base.html" %}

{% load i18n %}

{% block main %}

<h2>{% trans 'Username reminder' %}</h2>

<p>{% trans "Forgotten your username? Enter your email address below, and we'll email you your username." %}</p>

<form action="" method="post">{% csrf_token %}
{{ form.email.errors }}
<p><label for="id_email">{% trans 'Email address:' %}</label> {{ form.email }} <input type="submit" value="{% trans 'Send me my username' %}" /></p>
</form>

{% endblock %}
