<span data-dojo-attach-point="dctextarea">
		<label for="textual_body">Text</label>&nbsp;
		<textarea id="textual_body" data-dojo-attach-point="textual_body" data-dojo-type="dijit/form/Textarea"></textarea>
</span>					
