{% extends "registration/registration.html" %}

{% load i18n %}

{% block registration_header %}

<div>
	<h2>{% trans "Your OpenID is verified! " %}</h2>
	{% blocktrans %}
	<p>Your OpenID can now be associated with a new or existing membership. You can change the association later in your preferences.</p>
	{% endblocktrans %}
	<br /><br />
	<h3>{% trans "Associate your OpenID" %}</h3>
	{% blocktrans %}
	<p>If you're joining <strong>CHARMe</strong>, associate your OpenID with a new account. If you're already a member, associate with your existing account.</p>
	{% endblocktrans %}
</div>
	
<hr class="clear" />

<h4>A new account</h4>

{% endblock %}

{% block registration_footer %}

<hr class="clear" />
	
<div class=loginform>
	<h4>An existing account</h4>
	{% if auth_form.non_field_errors %}
	<div class=error>
		<ul>
			<li>{{ auth_form.non_field_errors }}</li>
		</ul>
	</div>
	{% endif %}	
	<div class=accounts>
        <form action="{% url 'user_register' %}" method="POST">
			{% csrf_token %}
			{% if request.REQUEST.next %}
				<input type="hidden" name="next" value="{{ request.REQUEST.next }}" />
			{% else %}
				<input type="hidden" name="next" value="{{ next }}" />
			{% endif %}
			<table>
				<tr>
					<td><label> {% trans "Username:" noop %} </label></td>
					<td>{{ auth_form.username }}</td>
					<td class=error>{{ auth_form.username.errors }}</td>
				</tr>
				<tr>
					<td><label> {% trans "Password:" noop %} </label></td>
					<td>{{ auth_form.password }}</td>
					<td class=error>{{ auth_form.password.errors }}</td>
				</tr>
				<tr>
					<td></td>
					<td style="text-align: right;">
						<input type="submit" value="Verify"></td>
					<td></td>
				</tr>
			</table>
		</form>
	</div>	
</div>
	
{% endblock %}
