{% extends "charme_base.html" %}

{% load i18n %}

{% block main %} 

<h2>{% trans 'Password reset complete' %}</h2>

<p>{% trans "Your password has been set. You may go ahead and log in now." %}</p>

<p>
	<a href="{{ login_url }}">{% trans 'Log in' %}</a>
</p>

{% endblock %}

