{% extends "charme_base.html" %}

{% load i18n %}

{% block main %}

{% if validlink %}

<h2>{% trans 'Enter new password' %}</h2>

<p>{% trans "Please enter your new password twice so we can verify you typed it in correctly." %}</p>

<form action="" method="post">{% csrf_token %}
<div class=contactform id="password_confirm">
	<div class=accounts>
		<table>
			<tr>
				<td><label>{% trans "New password:" noop %}</label></td>
				<td>{{ form.new_password1 }}</td>
				<td class=error>{{ form.new_password1.errors }}</td>
			</tr>
			<tr>
				<td><label>{% trans "Confirm password:" noop %}</label></td>
				<td>{{ form.new_password2 }}</td>
				<td class=error>{{ form.new_password2.errors }}</td>
			</tr>
			<tr>
				<td></td>
				<td style="text-align: right;"><input type="submit"
						value="{% trans 'Change my password' %}"></td>
				<td></td>
			</tr>
		</table>
	</div>
</div>
</form>

{% else %}

<h2>{% trans 'Password reset unsuccessful' %}</h2>

<p>{% trans "The password reset link was invalid, possibly because it has already been used.  Please request a new password reset." %}</p>

{% endif %}

{% endblock %}
